/*******************************************************************/
/*                                                                 */
/*   COMMON header file for IP-VLBI (K5/VSSP) utilities            */
/*          chead.h                                                */
/*                                                                 */
/*      Version:  1.01  2005-09-27                                 */
/*                1.02  2006-11-10  #define pclose(_Z) _pclose(_Z) */
/*                1.03  2006-12-08  Add CYGWIN case                */
/*                1.04  2007-02-14  Add u_int8_t type definition   */
/*                                  for WIN32 case                 */
/*                1.05  2007-06-27  Max scan# changed from 2000 to */
/*                                  5000                           */
/*                1.06  2007-07-20  Max scan# changed from 5000 to */
/*                                  3000                           */
/*                1.07  2009-09-17  #pragma warning (disable:4996) */
/*                                  to suppress warning in Win32   */
/*                1.08  2009-11-13  Max DAS number MAXDAS was      */
/*                                  added and set to 70            */
/*                                  Max statioin# changed from 30  */
/*                                  to 50                          */
/*                1.09  2013-01-29  Max freqtable number MAXCODES  */
/*                                  was added and set to 10        */
/*                1.10  2014-06-03  Add proper large file handling */
/*                                  for WIN32                      */
/*                1.11  2016-09-16  no more use of MAXSTAR, MAXOBS */
/*                                  MAXSTAT, MAXDAS, MAXCODES      */
/*                                                                 */
/*******************************************************************/
#ifndef _IP_CHEAD_
#define _IP_CHEAD_

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __CYGWIN__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#ifdef linux
#ifdef _K5_USE_FOPEN64_
#define fopen(_Z,_X) fopen64(_Z,_X)
#endif
#ifdef _LARGEFILE_SOURCE                      // added on 2014-06-03
#define fseek(_Z,_X,_W) fseeko(_Z,_X,_W)      // added on 2014-06-03
#define ftell(_Z) ftello(_Z)                  // added on 2014-06-03
#endif                                        // added on 2014-06-03
#endif

//#include "libipvlbi.h"
//#include "libsked.h"

/*  graphic display set */
//#ifndef PGDISP
// #define PGDISP "PGDISP"    // PGPLOT display device environment
//#endif
/*  graphic display device default set */
//#ifndef PGDISP_DFLT
// #ifdef WIN32
//  #define PGDISP_DFLT "/GW"      
// #else
//  #define PGDISP_DFLT "/XTERM"
// #endif
//#endif


/* constant set */

#ifndef FILEMAXSIZE
#define FILEMAXSIZE  2000000000   //  File maximum limit size in bytes when limiting
#endif

#ifndef PIRA
#define PIRA   0.017453292519943295769236907684886    /* PI/180  degree to radian */
#endif
#ifndef PIDE
#define PIDE   57.295779513082320876798154814105      /* 180/PI radian to degree  */
#endif
#ifndef TWOPI
#define TWOPI  6.283185307179586476925286766559       /* 2PI                      */
#endif
#ifndef PI
#define PI     3.1415926535897932384626433832795      /* PI                       */
#endif
#ifndef HALFPI
#define HALFPI 1.5707963267948966192313216916398      /*  PI/2                    */
#endif
#ifndef VLIGHT
#define VLIGHT 2.99792458E+8    /* speed of light (m/sec)  */
#endif

/* mainly used in libapri etc */
#ifndef CONVA
#define CONVA 4.8481368110953599358991410235795e-6    /* PI/(3600*180)  arcsec to radian */
#endif
#ifndef CONVS
#define CONVS 7.2722052166430399038487115353692e-5    /* PI/(3600*12) time sec to radian */
#endif
#ifndef CONVD
#define CONVD 0.017453292519943295769236907684886    //*PI/180  degree to radian  */
#endif
#ifndef CONVH
#define CONVH 0.26179938779914943653855361527329    /* PI/12 hour to radian  */
#endif

/* schedule file elements set */
// no more use 2016-09-16
/*
#ifndef MAXSTAR
#define MAXSTAR 1500            // Maximum star numbers in schedule file
#endif

#ifndef MAXOBS
#define MAXOBS 3000            // Maximum observation (scan) numbers in schedule file
//#define MAXOBS 2250            // Maximum observation (scan) numbers in schedule file
#endif

#ifndef MAXSTAT
//#define MAXSTAT 30            // Maximum station numbers in schedule file
#define MAXSTAT 50            // Maximum station numbers in schedule file 2009-11-13
#endif


#ifndef MAXDAS               // added on 2009-11-13
#define MAXDAS 70            // Maximum data acquisition numbers in VEX file
#endif

#ifndef MAXCODES                // added on 2013-01-28
#define MAXCODES 10          // Maximum number of code (frequency tables) in a SKED schedule file
#endif
*/

/* for WIN32 */

#ifdef WIN32
#pragma warning (disable:4996)
#ifndef _FORWIN32SET_
#define _FORWIN32SET_
typedef unsigned int    u_int32_t;
typedef unsigned int    uint32_t;
typedef unsigned short int    u_int16_t;
typedef unsigned short int    uint16_t;
typedef unsigned char   u_int8_t;
typedef _int64 int64_t;
//typedef long long int int64_t;                   // thi is also OK for WIN 32 2015-11-04
#define popen(_Z,_X) _popen(_Z,_X)
#define fseek(_Z,_X,_W) _fseeki64(_Z,_X,_W)      // added on 2014-06-03
#define pclose(_Z) _pclose(_Z)
#define ftell(_Z) _ftelli64(_Z)                  // added on 2014-06-03
#endif
#endif



#endif
