/********************************************************************/
/*                                                                  */
/*   Program Name :  speana2                                        */
/*                                                                  */
/*      Version:  2015-10-28                                        */
/*                                                                  */
/*                                                                  */
/*      Copyright (c) 2003-2015 T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*
 ho|ukah{[h擾f[^̃XyNg ̂Q
        iItCj

          T.Kondo 2003-04-12
                  2003-04-21  sekibun𕉂̒lƂƌJԂԊuƂ悤ɂ
                  2005-06-12  update due to update in common header name
                  2005-09-27  Allow to use PGDISP as an environmetal variable
                                   for PPGPLOT device
                  2006-01-31  support K5/VSSP32 format
                              allow to use FFTW3.0 package

                  2007-02-22  update due to libvssp32.h update
                  2007-10-26  leave window open in case of Windows
                  2007-11-02  support GNUPLOT
                  2007-11-04  improve appearance in case of GNUPLOT postscript
                  2008-02-19  allow file name starting with '0' and having more than 
                              equal to 2 characters.
                              option '--help' and '--ver[sion]' were added
                  2011-02-07 "long int" type is changed to "int" type
                  2014-09-11  minor debug
                  2014-19-29  add parameters (plot amplitude range)
                  2015-10-23 support AUX field format #22
                  2015-10-28 update due to "libspeplot" being an independent library

    点 speana2 filename [smode [sekibun1 [sekibun [soffset [comment [f1khz [f2khz [mindbm maxdbm]]]]]]]]
          filename -- data file name
                            0 for "tds.data" 
               smode -- frequency resolution mode
                       0: 4096@(about 1kHz@4MHz sampling)
                       1: 4194304 (about Hz@4MHz sampling)
               sekibun1  -- unit integration period (integer sec)
                            default is 1 (sec)
               sekibun  --  total integration period (integer sec)
                            default is 300 (sec)
                            -VE means repeating period (sec)
               soffset  -- start offset time (sec)
                            default 0 sec
               comment  -- comment displayed at the top of graph.  " "(space) cannot be included.
                           If omitted, you will be asked to input.  This case " "(space) can be 
                           included in a comment.
               f1khz  ---  lower frequency of power max search and plot range (kHz).  default is 0
               f2khz  ---  highest frequency of power max search and plot range (kHz).  default is max video frequency
               mindbm  ----  display amplitude minimum (dBm) (default automatic)\n");
               maxdbm  ----  display amplitude maximum (dBm) (default automatic)\n");
               outfile ----  output file name (default is speana2.txt)

 D:\IPVLBI\data\k5vssp32\VSSP32sample.dat 0 1 1 0 NONE 900 1100
*/
#define SPEANA2_VERSION "Ver. 2015-10-28"

//#define USE_GNUPLOT    // use GNUPLOT  (default is use of PGPLOT)
#include "g_head.h"    // header for graphics

#ifdef WIN32
  #define USE_FFTW    // use FFTW package
#endif

#ifdef USE_FFTW
 #ifdef WIN32
  #include "fftw3.h"     // use of FFTW package
 #else
  #include <fftw3.h>     // use of FFTW package
 #endif
#endif

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libk5.h"      // common header for IP-VLBI utilities
#include "libfft.h"

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#include "libvssp32.h"
#include "libspeplot.h"

/*    Offset from dB to dBm conversion from A/D multibit sampling data */ 
#define OFFSET1  16.0     //1 bit sampling full power convert to 0 dB (theoretical)
      /* following constants are provisional  (measured on 2002/5/18) */
#define OFFSET2   9.0     //2 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET4  -1.0     //4 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET8 -26.0     //8 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET16 -74.0     //16 bit sampling data offset (this is for USBADC adjusting value) 2015-10-23

#ifdef WIN32
int tempcounter;  // counter for tempfile manage
#endif

//#include "libspeplot.c"


int main(int argc, char *argv[])
{
    int nloop,nloop1,sekibun,sekibun1,sekibun1st,npmax;
    int i,imax, m, k, n, kaisu;
    int smode,usampl,nfft;
    //int kword;
	int kbytes;
    //unsigned char *ibuf;
    int nbyte=5000;
    FILE *stream;
    //int bytesread;
    int bits, bytes, numb;
    //int mode=0;   // log frequency scale
    //int mode=1;   // linear frequency scale
    int scount[4];
	int nsecs;
    float fsampl,tsampl,fact;
    //double tinteg,tusampl;
    double tusampl;
    //int *idata;
    double *cdata;
    double *f_hanning;
    double theta;
    float *xdata, *ydata, *fdat;
    double sum,dc;
    double tinteg,tinteg1;
    float pows;
    float pmax,pmin,pmaxdbm,pmindbm,cofst,df;
    float ymaxdb,ymindb;
    double nfact;
#ifdef USE_FFTW
    fftw_complex *cc;
    fftw_plan pfcc, pbcc;
#endif
    char ltime1st[60];
    char pgdevice_dflt[20];
	char* dflt_fname="tds.data";
    char outfile[120];    // 2014-10-29
    K5VSSP_file_t K5;
    speana_param_t st;

    /* set graphic device */
    strcpy(st.ps_fname,PS_OUT_FILE);
#ifdef USE_GNUPLOT   // using gnuplot
    strcpy(st.g_display,G_DISPLAY);
    strcpy(pgdevice_dflt,G_DISPLAY);
    strcpy(st.g_ps_device,G_PS_DEVICE);
#else
    strcpy(st.g_display,select_env_dflt(PGDISP, PGDISP_DFLT));
    strcpy(pgdevice_dflt,st.g_display);
    sprintf(st.g_ps_device,"%s%s",st.ps_fname,G_PS_DEVICE);
#endif

    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
       printf("speana2 %s\n",SPEANA2_VERSION);
#ifdef USE_FFTW
       printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
       printf(" compliled for GNUPLOT \n");
#else
       printf(" compliled for PGPLOT \n");
#endif
       printf("Usage: speana2 filename [smode [sekibun1 [sekibun [soffset [comment [f1khz [f2khz [mindbm [maxdbm]]]]]]]]]\n");
       printf("\n");
       printf("         where  filename ---- Data file name\n");
       printf("                              if 0 is given, tds.data will be used\n");
       printf("                smode -------- process mode\n");
       printf("                              =0 : 4096 points FFT (default)\n");
       printf("                                   (about 1 kHz resolution @4MHz sampling)\n");
       printf("                              =1 : 4194304 points FFT (about 1Hz resolution)\n");
       printf("                sekibun1 --- unit Averaging period (integer seconds)\n");
       printf("                             default is 1\n");
       printf("                sekibun ---- total averaging period\n");
       printf("                             -VE means repeating period\n");
       printf("                             default is 300\n");
       printf("                soffset --- Start time offset in sec (integer seconds)\n");
       printf("                             default is 0\n");
       printf("                comment --- Comment appeared on a graph\n");
       printf("                            if omitted, you will be asked later.\n");
       printf("                f1khz   ----  search frequency range (lower boundary) (kHz)\n");
       printf("                              for power Maximum (default is 0)\n");
       printf("                f2khz   ----  search frequency range (upper boundary) (kHz)\n");
       printf("                              for power Maximum (default is video frequency)\n");
       printf("                mindbm  ----  display amplitude minimum (dBm) (default auto)\n");
       printf("                maxdbm  ----  display amplitude maximum (dBm) (default auto)\n");
       printf("                outfile ----  output file name (default speana2.txt)\n");
       printf("\n");
#ifndef USE_GNUPLOT
       printf("  Note: %s ---  environment variable for PGPLOT display device\n",PGDISP);
       printf("                   = %s",getenv(PGDISP));
       if(getenv(PGDISP)==NULL) {
           printf(",  program deflt is ( %s )\n",PGDISP_DFLT);
       } else {
           printf("\n");
       }
#endif
       printf("\n");
	   exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
       printf("speana2 %s\n",SPEANA2_VERSION);
#ifdef USE_FFTW
       printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
       printf(" compliled for GNUPLOT \n");
#else
       printf(" compliled for PGPLOT \n");
#endif
	   exit(0);
    }

    strcpy(st.fname,*(argv+1));
    if (strlen(*(argv+1))==1 && st.fname[0]=='0'){  // 2008-02-19
       strcpy(st.fname,dflt_fname);
    }
    //X.fname=fname;
    strcpy(K5.fname,st.fname);
	
	if (argc>=3){
	    smode = atoi(*(argv + 2 ));
    } else {
        smode=0;
    }

	if (argc>=4){
	   sekibun1 = atoi(*(argv + 3 ));
       st.sekibun1=atof(*(argv + 3 ));
    } else {
       sekibun1=1;     // default unit integration time (sec)
       st.sekibun1=1.0;
    }

	if (argc>=5){
	   sekibun = atoi(*(argv + 4 ));
	   st.sekibun = atof(*(argv + 4 ));
	} else {
       sekibun=300;     
       st.sekibun=300.0;     
    }

	if (argc>=6){
	   st.soffset = (int)atoi(*(argv + 5 ));
	} else {
       st.soffset=0;     // default startoffset
    }

    printf("speana2 %s",SPEANA2_VERSION);
#ifdef USE_FFTW
    printf(" compliled with FFTW3.0 \n");
#else
    printf("\n");
#endif

    printf("Data File is %s\n",K5.fname);
    K5.size=get_file_size(K5.fname);
	if( (K5.stream  = fopen( K5.fname, "rb" )) == NULL ){
       printf("File Open Error! \n");
	   goto exitend2;
    }
    k5vssp_t_ini(&K5);

    /* get comment */

	if (argc>=7){
        strcpy(st.comment,*(argv+6));
    } else {
        /* enter board ID */
        /*
        printf("Enter IP-VLBI BOARD ID -> ");
        read1line(iplab);      // 1 line read from stdin
        printf("Board ID is %s\n",iplab);      
        strcpy(lab2,"IP-VLBI Board ID : ");
        strcat(lab2,iplab);
        */

        /* enter any comment */

        printf("Enter any comment -> ");
        read1line(st.comment);      // 1 line read from stdin
        //printf("Board ID is %s\n",iplab);      
        //strcat(lab2,iplab);
    }
	
	if (argc>=8){
	   st.f1khz = atof(*(argv + 7 ));
	} else {
	   st.f1khz=0.0;
    }
	if (argc>=9){
	   st.f2khz = atof(*(argv + 8 ));
    } else {
	   st.f2khz=0.0;
    }
	if (argc>=10){
        st.mindbm = atof(*(argv + 9 ));
	} else {
	   st.mindbm=0.0;
    }
	if (argc>=11){
	   st.maxdbm = atof(*(argv + 10 ));
    } else {
	   st.maxdbm=0.0;
    }
	if (argc>=12){
        strcpy(outfile,*(argv+11));
    } else {
        strcpy(outfile,"speana2.txt");
    }
	printf("f1khz,f2khz %f %f\n",st.f1khz,st.f2khz);
	printf("mindbm,maxdbm %f %f\n",st.mindbm,st.maxdbm);
    /* Data header check to get sampling parameters */
    if((kbytes=CheckK5header_with_Read(&K5)) < 0){
        printf("File read error! \n");
        //fgetpos(K5.stream,&K5.pos); 
        i=fclose( K5.stream ) ;
        goto exitend;
    }
		
	
	st.amode=0; st.mode=1;   // lenear scale and spectrum mode
	st.pmode=0;


    /* parameter set */
    st.sfreq=K5.sfreq;
    st.adbit=K5.adbit;
    st.numch=K5.numch;
    fsampl=(float)(1000.0*(float)st.sfreq);   // sampling frequency (Hz)
    tsampl=(float)(1.0/fsampl);            // sampling period (sec)
    //bits=((long)fsampl)*st.adbit*st.numch;  // # of bits in 1 sec data except header
    bits=((int)fsampl)*st.adbit*st.numch;  // # of bits in 1 sec data except header
    bytes=bits/8;                     // # of bytes in 1 sec data except header
    K5.fsample=fsampl;
    //sekibun=1;                       // Averaging period for Spectrum (sec)
    K5.bytesec=bits/8;                  //# of bytes in 1 sec data

    //smode=1;                         // low resolution mode

	usampl=4000;
    //if(smode ==1){ usampl=10000; }
    if(smode ==1) usampl=4000000; 
    //    else {
    //       usampl=50000;
    //       if(sfreq == 40){ usampl=40000; }
    //       if(sfreq == 100){ usampl=100000; }
    //       if(sfreq == 200){ usampl=200000; }
    //    }
    imax=(st.sfreq*1000)/usampl;
    numb=usampl*st.adbit*st.numch/8;   // # of bytes in unit # of samples
    K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
    K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second

    tusampl=(double)usampl/K5.fsample;   // period of usampl

    printf("imax numb  %d %d\n",imax,numb);    //debug

    //    nfft=1024;
    //    if(usampl < 524288){nfft= 524288;}
    //    if(usampl < 262144){nfft= 262144;}
    //    if(usampl < 131072){nfft= 131072;}
    //    if(usampl < 65536){nfft= 65536;}
    //    if(usampl < 32768){nfft= 32768;}
    //    if(usampl < 16384){nfft= 16384;}
    nfft=4096;
	if(smode==1) nfft=4194304;

    st.nfft=nfft;

    nfact=2.0/(double)nfft;

    K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl
    K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

    K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer
    K5.idata  = (int *)malloc(32*K5.nword*sizeof(int)); // K5 data read buffer

#ifdef USE_FFTW
    cc=fftw_malloc(sizeof(fftw_complex)*nfft);

    pfcc=fftw_plan_dft_1d(nfft, cc, cc, FFTW_FORWARD, FFTW_ESTIMATE);
    pbcc=fftw_plan_dft_1d(nfft, cc, cc, FFTW_BACKWARD, FFTW_ESTIMATE);
#endif
	
	printf("usampl,nfft %d %d\n",usampl,nfft);
//        ibuf = (unsigned char *)malloc(numb);
//        idata = (int *)malloc(4*usampl*sizeof(int));
    cdata = (double *)malloc(2*nfft*sizeof(double));
    st.spows = (float *)malloc(4*nfft*sizeof(float));
    xdata = (float *)malloc(nfft*sizeof(float));
    ydata = (float *)malloc(nfft*sizeof(float));
    npmax=nfft/2;

    fdat   = (float *)malloc(npmax*sizeof(float));
    //autocor=(float *)malloc(4*nfft*sizeof(float));
    df=fsampl/(float)nfft;
    for(n=0; n<npmax; n++) fdat[n]=(float)(n*df/1000.0); 

    f_hanning = (double *)malloc(nfft*sizeof(double));

    //Hanning filter coeeficients prepare
    for(k=0; k<nfft; k++){
        theta=360.0*(double)k/(double)(nfft-1) - 180.0;
        theta=fmod(theta,360.0) ;
        f_hanning[k]=0.5*(1.0+cos(theta*PIRA));
    }
      

    fact=((float)usampl)*((float)usampl);
        
    /* sampling frequency in text for print */
//    if(sfreq < 1000){
//        sprintf(labfsampl,"%dkHz",sfreq);
//    } else {
//        sprintf(labfsampl,"%dMHz",sfreq/1000);
//    }


    /* foward startffset seconds */
    if(fwdNsecK5((int)st.soffset, &K5) <0){            // 2014-09-11
        printf("Foward error %d\n",(int)st.soffset);   // 2014-09-11
        //i=fclose( K5.stream ) ;
        goto exitend;
    }


    printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",K5.hh,K5.mm,K5.ss,K5.seconds);
    sprintf(ltime1st,"Time(UT) %02d:%02d:%02d",K5.hh,K5.mm,K5.ss);
    strcpy(st.lab2,ltime1st);
    strcat(st.lab2,"  ");
    strcat(st.lab2,st.comment);
        
	if( (stream  = fopen( outfile, "w" )) == NULL ){
         printf("File Open Error! %s\n",outfile);
	     goto exitend2;
    }

    tinteg1=0.0;   // not used yet
    tinteg=0.0;    // not used yet

	//sekibun1=30;
	//sekibun1=1;
	nloop1=0;
	
	fprintf(stream,"%d %d %d\n", st.nfft, st.sfreq, st.adbit);
	//fprintf(stream,"%d %d\n",sekibun1,sekibun);
	fprintf(stream,"%f %f\n",st.sekibun1,st.sekibun);
	
	nloop=0;  //  sekibun (sec)
	sekibun1st=1;
loop_main:
    for(k=0; k<4*nfft; k++) st.spows[k]=0.0;
    for(k=0; k<4; k++) scount[k]=0;
loop1:
    if(nloop >0) {
        if((kbytes=CheckK5header_with_Read(&K5)) < 0){
            i=fclose( K5.stream ) ;
            goto exitend;
        }
        printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
                K5.hh,K5.mm,K5.ss,K5.seconds);
    }
    if(sekibun1st==1){
      	sprintf(ltime1st,"Time(UT) %02d:%02d:%02d",K5.hh,K5.mm,K5.ss);
        strcpy(st.lab2,ltime1st);
        strcat(st.lab2,"  ");
        strcat(st.lab2,st.comment);
        sekibun1st=0;
    }
    for(i=0; i< K5.imax; i++){
        K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
        K5.pos+=K5.wordread*4;
        K5.vflag=1; // validiate data
        parseADdata_3N(&K5);
        for(m=0; m<st.numch; m++){
            kaisu=0;
            sum=0.0;

            // initialize array
#ifdef USE_FFTW
            for(k=0; k<nfft; k++){ cc[k][0]=0.0; cc[k][1]=0.0;}
#else
            for(k=0; k<2*nfft; k++) cdata[k]=0.0;
#endif

           	for(k=0; k<K5.npos/K5.numch; k++) {
                /* set data */
#ifdef USE_FFTW
                cc[k][0]=(double)K5.idata[k*st.numch+m];  
                sum+=cc[k][0];
#else
                cdata[2*k]=(double)K5.idata[k*st.numch+m];  
                sum+=cdata[2*k];
#endif
                kaisu++;
            }
             
            dc=sum/(double)kaisu;    // DC component
     
            //printf("m DC = %d %f\n",m,dc);  //debug
            // DC offset removal from data
            for(k=0; k<K5.npos/st.numch; k++){
#ifdef USE_FFTW
                cc[k][0]-=dc;
#else
                cdata[2*k]-=dc;
#endif
            }
            //hanning(cdata,nfft);  // hanning filter
            //cdft(2*nfft,-1,cdata);
            if(st.amode ==0) {    // spectrum mode
                //hanning(cdata,usampl);  // hanning filter
                //cdft(2*nfft,1,cdata);
                for(k=0; k<nfft; k++){
#ifdef USE_FFTW
                   cc[k][0]*=f_hanning[k];  // hanning filter
                   cc[k][1]*=f_hanning[k];  // hanning filter
#else
                   cdata[2*k]*=f_hanning[k];
                   cdata[2*k+1]*=f_hanning[k];
#endif
                }
#ifdef USE_FFTW
                fftw_execute(pbcc);
#else
                cdft(2*nfft,1,cdata);
#endif
            } else {
#ifdef USE_FFTW
                fftw_execute(pfcc);
                for(k=0; k<nfft; k++){
                    cc[k][0]*=nfact; cc[k][1]*=nfact;
                }
#else
                cdft(2*nfft,-1,cdata);
#endif
            }
            for(k=0; k<nfft; k++) {
#ifdef USE_FFTW
                pows=(float)((cc[k][0]*cc[k][0]+cc[k][1]*cc[k][1])/(double)fact);
#else
                pows=(float)((cdata[2*k]*cdata[2*k]+cdata[2*k+1]*cdata[2*k+1])/(double)fact);
#endif
                st.spows[k*4+m]=st.spows[k*4+m]+pows;
            }

            scount[m]++;
        }

    }
    nloop++;
    nloop1++;
    if(nloop1 < sekibun1) goto loop1;
	nloop1=0;

    if(st.amode > 0) {   // auto correlation function calc
       for(m=0; m<st.numch; m++){
          for(k=0; k<nfft; k++){
              st.spows[k*4+m]/=scount[m]; // nomalization
#ifdef USE_FFTW
              cc[k][0]=st.spows[k*4+m]; cc[k][1]=0.0;
#else
              cdata[2*k]=st.spows[k*4+m];
              cdata[2*k+1]=0.0;
#endif
          }
          if(m==1 || m==3){
#ifdef USE_FFTW
              cc[0][0]=1.0;
#else
              cdata[0]=1.0;  // to avoid math error
#endif
          }
#ifdef USE_FFTW
          fftw_execute(pbcc);
#else
          cdft(2*nfft,1,cdata);
#endif
          for(k=0; k<nfft; k++){
              // if(k<10) printf("%d %e\n",k,cdata[2*k]);  // debug
              // if(k>nfft-10) printf("%d %e\n",k,cdata[2*k]);  // debug
#ifdef USE_FFTW
              if(k==0) pmax=(float)cc[0][0];
              if(cc[k][0] > pmax) pmax=(float)cc[k][0];
              st.spows[k*4+m]=(float)cc[k][0];
#else
              if(k==0) pmax=(float)cdata[0];
              if(cdata[2*k] > pmax) pmax=(float)cdata[2*k];
              st.spows[k*4+m]=(float)cdata[2*k];
#endif
          }
          for(k=0; k<nfft; k++){  // normalize
              st.spows[k*4+m]/=pmax;
              // if(k<10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
              // if(k>nfft-10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
          }
             
       }
         
    } else {
       	/* convert power to dB  then dBm */       
       	cofst=OFFSET1;       //1 bit sampling full power convert to 0 dB
       	if( st.adbit==8){cofst=OFFSET8; }    // 8bit sampling offset
       	if( st.adbit==4){cofst=OFFSET4; }    // 4bit sampling offset
       	if( st.adbit==2){cofst=OFFSET2; }    // 4bit sampling offset
        if( st.adbit==16) cofst=OFFSET16;     // 16bit sampling offset (arbitrary value) added on 2015-10-23


       	for(m=0; m<st.numch; m++){
          		for(k=0; k<nfft; k++){
              		st.spows[k*4+m]/=scount[m]; // nomalization
              		st.spows[k*4+m]=(float)(10.0*log10(st.spows[k*4+m]) + cofst); 
             		if(k==0){
               	 	    pmaxdbm=st.spows[k*4+m];
               		    pmindbm=st.spows[k*4+m];
                        if(m==0){
                           ymaxdb=pmaxdbm;
                           ymindb=pmindbm;
                        }
                    } else {
                	    if(st.spows[k*4+m] > pmaxdbm) pmaxdbm=st.spows[k*4+m]; 
                	    if(st.spows[k*4+m] < pmindbm) pmindbm=st.spows[k*4+m]; 
                    }
                }
                pmax=pmaxdbm-cofst;
           	    pmin=pmindbm-cofst;
           	    printf("CH# %d: Maximum data (dB, dBm) is %f %f\n", m+1,pmax,pmaxdbm);      
                if(pmaxdbm > ymaxdb) ymaxdb=pmaxdbm;
                if(pmindbm < ymindb) ymindb=pmindbm;
        }
        printf("All CH: Maximum and Minimum data (dBm) are %f %f\n", ymaxdb, ymindb);      

    }

    /* write to text file */
    for(m=0; m<st.numch; m++){
        fprintf(stream,"%s\n",st.lab2);
        //for(k=0; k<nfft; k++) xdata[k]=st.spows[k*4+m];
        for(k=0; k<npmax; k++){
           fprintf(stream,"%d %f %f\n",k+1,fdat[k],st.spows[k*4+m]);
        }
    }

    /* plot spectrum (or auto cor) POSTSCRIPT OUT and XTERM*/

    st.sekibun=(double)sekibun;
    plotspe2(&st);

	if(nloop < sekibun) {
		sekibun1st=1;
		goto loop_main;
	}
	if(sekibun < 0) {  //  -sekibun is repeating period
	   nsecs=-sekibun-sekibun1-1;  
	   sekibun1st=1;
       /* foward startffset seconds */
	   if(nsecs > 0) {  // 1st data are not sync block!! so nsecs = -sekibun-sekibun "-1"
          //if(fwdNsec(ibuf, nsecs, numb, imax, &X) <0)  goto exitend;
          if(fwdNsecK5(nsecs, &K5) <0){
             printf("Foward error %d\n",nsecs);
             goto exitend;
          }
      } 
	   goto loop_main;
    }
 exitend: ;
 	i=fclose( K5.stream ) ;
    i=fclose(stream); 
    free(K5.ubuf);
    free(K5.idata);
    free(cdata);
    free(st.spows);
    free(xdata);
    free(ydata);
    free(fdat);

    printf("Output file is %s\n",outfile);

#ifdef USE_FFTW
    fftw_destroy_plan(pfcc);
    fftw_destroy_plan(pbcc);
    fftw_free(cc);
#endif

 exitend2: ;

       return 0;
}
