/*******************************************************************/
/*                                                                 */
/*   File name :  speana_n.h                                       */
/*                                                                 */
/*      Version:   2015-11-02                                      */
/*                                                                 */
/*   Copyright (c) 2015-     T.Kondo All Right Reserved            */
/*                                                                 */
/*******************************************************************/

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __CYGWIN__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#include "g_head.h"    // header for graphics

#ifdef WIN32
  #define USE_FFTW    // use FFTW package
#endif

#ifdef USE_FFTW
 #ifdef WIN32
  #include "fftw3.h"     // use of FFTW package
 #else
  #include <fftw3.h>     // use of FFTW package
 #endif
#endif

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libfft.h"

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#include "libvssp32.h"
#include "libspeplot.h"


typedef struct runp     // run parameters for this program
{
    char fname[128];  // data file name
    char comment[60]; // comment
    double sekibun;   // given integration period
    double soffset;   // start offset in floating second 
    double f1khz;     // search frequency range (lower boundary) (kHz)
    double f2khz;     // search frequency range (upper boundary) (kHz)
    double mindbm;    // plot amplitude range
    double maxdbm;    //     "
    int  mode2;        // axis mode
    int  mode;        // axis mode
    int  ymode;       //
    int amode;        // 
    int pmode;        // plot mode
    int flag_halt;
    int sleep_msec;
} runp_t;

typedef struct fftp     // FFT parameters
{
    int flag_fftw;    // FFTW use flag  1: use 0: not use
    int nfft;
    double *cdata;
#ifdef USE_FFTW
    fftw_complex *cc;  //=> global
    fftw_plan pfcc, pbcc;  // => global
#endif
    double *f_hanning;
} fftp_t;